VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Base64"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'****************************************************************************
'<comment>
' <summary>
'  Class module that contains the methods that do Base64 coding and decoding
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2003-07-28</date>
'</comment>
'****************************************************************************
Option Explicit
'****************************************************************************
'<comment>
' <summary>
' Decodes a base-64 encoded string (BSTR type).
' </summary>
' <remarks></remarks>
' <param name="base64String">
'  The base64 string that is to be decoded.
'  transaction.
' </param>
'</comment>
'****************************************************************************
Static Function Decode(ByVal base64String As String) As String
  'rfc1521
  '1999 Antonin Foller, PSTRUH Software, http://pstruh.cz
  Const base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
  Dim dataLength As Integer
  Dim sOut As String
  Dim groupBegin As Integer
    
  'remove white spaces, If any
  base64String = Replace(base64String, vbCrLf, "")
  base64String = Replace(base64String, vbTab, "")
  base64String = Replace(base64String, " ", "")
  
  'The source must consists from groups with Len of 4 chars
  dataLength = Len(base64String)
  If dataLength Mod 4 <> 0 Then
    Decode = "" ' ("Base64Decode:Bad Base64 string.")
    Exit Function
  End If

  
  ' Now decode each group:
  For groupBegin = 1 To dataLength Step 4
    Dim numDataBytes As Integer
    Dim CharCounter As Integer
    Dim thisChar As String
    Dim thisData As Integer
    Dim nGroup As Long
    Dim sGroup As String
    Dim pOut As String
    
    ' Each data group encodes up To 3 actual bytes.
    numDataBytes = 3
    nGroup = 0

    For CharCounter = 0 To 3
      ' Convert each character into 6 bits of data, And add it To
      ' an integer For temporary storage.  If a character is a '=', there
      ' is one fewer data byte.  (There can only be a maximum of 2 '=' In
      ' the whole string.)

      thisChar = Mid(base64String, groupBegin + CharCounter, 1)

      If thisChar = "=" Then
        numDataBytes = numDataBytes - 1
        thisData = 0
      Else
        thisData = InStr(base64, thisChar) - 1
      End If
      If thisData = -1 Then
        Decode = " " ' ("Base64Decode: Bad character In Base64 string.")
        Exit Function
      End If

      nGroup = 64 * nGroup + thisData
    Next
    
    'Hex splits the long To 6 groups with 4 bits
    sGroup = Hex(nGroup)
    
    'Add leading zeros
    sGroup = String(6 - Len(sGroup), "0") & sGroup
    
    'Convert the 3 byte hex integer (6 chars) To 3 characters
    pOut = Chr(CByte("&H" & Mid(sGroup, 1, 2))) + _
      Chr(CByte("&H" & Mid(sGroup, 3, 2))) + _
      Chr(CByte("&H" & Mid(sGroup, 5, 2)))
    
    'add numDataBytes characters To out string
    sOut = sOut & Left(pOut, numDataBytes)
  Next

  Decode = sOut
End Function
'****************************************************************************
'<comment>
' <summary>
' Encodes a string to be base-64 encoded (BSTR type).
' </summary>
' <remarks></remarks>
' <param name="inData">
'  The string that is to be encoded.
'  transaction.
' </param>
'</comment>
'****************************************************************************
Static Function Encode(inData As String) As String
  'rfc1521
  '2001 Antonin Foller, PSTRUH Software, http://pstruh.cz
  Const base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
  Dim cOut As String
  Dim sOut As String
  Dim i As Integer
    
  'For each group of 3 bytes
  For i = 1 To Len(inData) Step 3
    Dim nGroup As Long
    Dim pOut As String
    Dim sGroup As String
    
    'Create one long from this 3 bytes.
    nGroup = &H10000 * Asc(Mid(inData, i, 1)) + _
      &H100 * MyASC(Mid(inData, i + 1, 1)) + MyASC(Mid(inData, i + 2, 1))
    
    'Oct splits the long To 8 groups with 3 bits
    sGroup = Oct(nGroup)
    
    'Add leading zeros
    sGroup = String(8 - Len(sGroup), "0") & sGroup
    
    'Convert To base64
    pOut = Mid(base64, CLng("&o" & Mid(sGroup, 1, 2)) + 1, 1) + _
      Mid(base64, CLng("&o" & Mid(sGroup, 3, 2)) + 1, 1) + _
      Mid(base64, CLng("&o" & Mid(sGroup, 5, 2)) + 1, 1) + _
      Mid(base64, CLng("&o" & Mid(sGroup, 7, 2)) + 1, 1)
    
    'Add the part To OutPut string
    sOut = sOut + pOut
    
    'Add a new line For Each 76 chars In dest (76*3/4 = 57)
    'If (I + 2) Mod 57 = 0 Then sOut = sOut + vbCrLf
  Next
  Select Case Len(inData) Mod 3
    Case 1: '8 bit final
      sOut = Left(sOut, Len(sOut) - 2) + "=="
    Case 2: '16 bit final
      sOut = Left(sOut, Len(sOut) - 1) + "="
  End Select
  Encode = sOut
End Function

Private Function MyASC(OneChar As String) As Integer
  If OneChar = "" Then
    MyASC = 0
  Else
    MyASC = Asc(OneChar)
  End If
End Function
