VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RETSChangePasswordTransaction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************************************
'<comment>
' <summary>
'  RETSChangePasswordTransaction class encapsulates the RETS Change Password transaction
'  It implements RETSTransaction interface.
'   See the RETSTransaction class for 'inherited' details.
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
Option Explicit
Implements RETSTransaction

'****************************************************************************
'<comment>
' <summary>
'  The private m_transaction object to which transactions methods and attributes are delegated.
' </summary>
'</comment>
'****************************************************************************
Private m_transaction As New RETSTransaction

Private m_oldPassword As String
Private m_newPassword1 As String
Private m_newPassword2 As String

Private encrypted As String
Private decrypted As String

Private cat As New Logger


'****************************************************************************
'<comment>
' <summary>
'  The initializer for the class. It is NOT IMPLEMENTED!!!
' </summary>
'</comment>
'****************************************************************************
Private Sub Class_Initialize()

'   not implemented!!!
Const E_NOTIMPL = &H80004001
Err.Raise E_NOTIMPL

    m_transaction.requestType = "ChangePassword"
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction property
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get transaction() As RETSTransaction
    Set transaction = m_transaction
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction oldPassword property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get oldPassword() As String
    oldPassword = m_oldPassword
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction oldPassword property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new oldPassword property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let oldPassword(RHS As String)
    m_oldPassword = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction newPassword1 property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get newPassword1() As String
    newPassword1 = m_newPassword1
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction newPassword1 property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new newPassword1 property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let newPassword1(RHS As String)
    m_newPassword1 = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction newPassword2 property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get newPassword2() As String
    newPassword2 = m_newPassword2
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction newPassword2 property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new newPassword2 property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let newPassword2(RHS As String)
    m_newPassword2 = RHS
End Property

'''''''''''''''''''''''''''''''''''''''''''''''''''''
'   'inherited' methods
'''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Property Get RETSTransaction_capabilityUrls() As Dictionary
    Set RETSTransaction_capabilityUrls = m_transaction.capabilityUrls
End Property

Private Property Set RETSTransaction_capabilityUrls(RHS As Dictionary)
    Set m_transaction.capabilityUrls = RHS
End Property

Private Property Get RETSTransaction_context() As Dictionary
    Set RETSTransaction_context = m_transaction.context
End Property

Private Property Set RETSTransaction_context(aContext As Dictionary)
    Set m_transaction.context = aContext
End Property

Private Property Get RETSTransaction_requestMap() As Dictionary
    Set RETSTransaction_requestMap = m_transaction.requestMap
End Property

Private Property Let RETSTransaction_requestType(RHS As String)
    m_transaction.requestType = RHS
End Property

Private Property Get RETSTransaction_requestType() As String
    RETSTransaction_requestType = m_transaction.requestType
End Property

Private Property Get RETSTransaction_response() As String
    RETSTransaction_response = m_transaction.response
End Property

Private Property Let RETSTransaction_response(RHS As String)
    m_transaction.response = RHS
End Property

Private Property Get RETSTransaction_responseMap() As Dictionary
    Set RETSTransaction_responseMap = m_transaction.responseMap
End Property

Private Property Set RETSTransaction_responseMap(aResponseMap As Dictionary)
    Set m_transaction.responseMap = aResponseMap
End Property

Private Property Let RETSTransaction_responseStatus(RHS As String)
    m_transaction.responseStatus = RHS
End Property

Private Property Get RETSTransaction_responseStatus() As String
    RETSTransaction_responseStatus = m_transaction.responseStatus
End Property

Private Property Get RETSTransaction_responseStatusText() As String
    RETSTransaction_responseStatusText = m_transaction.responseStatusText
End Property

Private Property Let RETSTransaction_responseStatusText(RHS As String)
    m_transaction.responseStatusText = RHS
End Property

Private Sub RETSTransaction_setKeyValuePairs(str As String)
    m_transaction.setKeyValuePairs (str)
End Sub

Private Property Get RETSTransaction_url() As String
    RETSTransaction_url = m_transaction.url
End Property

Private Property Let RETSTransaction_url(aUrl As String)
    m_transaction.url = aUrl
End Property

Private Sub RETSTransaction_Preprocess()
    Dim errMsg As String
    Dim UserName As String

    Call m_transaction.Preprocess

    UserName = m_transaction.context.Item("username")

    Call cat.debugLog("username=" + UserName)
    Call cat.debugLog("oldPassword=" + oldPassword)
    Call cat.debugLog("newPassword1=" + newPassword1)
    Call cat.debugLog("newPassword2=" + newPassword2)

    If (oldPassword = "" Or Not oldPassword = m_transaction.context.Item("password")) Then
        errMsg = "Old Password does not match."
    ElseIf ((newPassword1 = "" Or newPassword2 = "") _
        Or Not (newPassword1 = newPassword2)) Then
        errMsg = "New Passwords do not match."
    Else
        Dim pwd As String
        pwd = encryptPWD()
        m_transaction.requestMap.Item("PWD") = pwd
    End If

    If (errMsg <> "") Then
        Call cat.warnLog(errMsg)

        RETSTransaction_responseStatus = "20513" ' Miscellaneous error
        RETSTransaction_responseStatusText = errMsg
        RETSTransaction_response = errMsg

        errMsg = ""
    End If
End Sub

Private Sub RETSTransaction_Postprocess()
    Call m_transaction.Postprocess
        
    m_transaction.context.Item("password") = newPassword1
End Sub

Private Function encryptPWD() As String
    Dim source As String

'    byte [] key = makeKey();
'    source = newPassword1+":"+username;
'
'    encryptPWD = DES(key, source)
    encryptPWD = ""
End Function



