VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RETSConnection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************************************
'<comment>
' <summary>
'  RETSConnection class encapsulates the RETS HTTP Connection
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
Option Explicit

Private headerMap As New Dictionary 'Key value pairs for request header.
Private responseMap As New Dictionary 'Key value pairs for response header.
Private serverUrl As String
Private errMsg As String
Private isRetryingAuthorization As Boolean
Private wwwAuthenticate As String

Private USER_AGENT      As String
Private RETSVersion     As String
Private mClientPassword As String
Private mRequestID      As String
Private authenticateDictionary  As Dictionary

' User agent constant. (User defined string)
Private Const INTERNET_USER_AGENT = "VB_RETS_IDX/1.0"
Private Const DEFAULT_RETS_VERSION = "RETS/1.0"

'****************************************************************************
'<comment>
' <summary>
'   information that must be passed between the connection and a transaction
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private connectionContext As New Dictionary

Private cat As New Logger

'   wininet HANDLEs
Private hInternetSession As Long
Private hInternetConnect As Long
Private hHttpOpenRequest As Long

Public Property Get ClientPassword() As String
    ClientPassword = mClientPassword
End Property
Public Property Let ClientPassword(value As String)
    mClientPassword = value
End Property

Public Property Get requestId() As String
    requestId = mRequestID
End Property
Public Property Let requestId(value As String)
    mRequestID = value
End Property

Public Property Get userAgent() As String
    userAgent = USER_AGENT
End Property
Public Property Let userAgent(agent As String)
    USER_AGENT = agent
End Property


Public Property Get RETS_Version() As String
    RETS_Version = RETSVersion
End Property
Public Property Let RETS_Version(agent As String)
    RETSVersion = agent
End Property


'****************************************************************************
'<comment>
' <summary>
'  The initializer for the class. It opens an internet session.
' </summary>
'</comment>
'****************************************************************************
Private Sub Class_Initialize()
    hInternetSession = 0
    hHttpOpenRequest = 0
    hInternetConnect = 0
    
 ' the following values are the default values for the request.
    Me.userAgent = INTERNET_USER_AGENT
    Me.RETS_Version = DEFAULT_RETS_VERSION

   InitializeSession
    
End Sub

Private Sub InitializeSession()

    hInternetSession = InternetOpen(Me.userAgent, _
                            INTERNET_OPEN_TYPE_PRECONFIG, _
                            vbNullString, vbNullString, 0)

End Sub

'****************************************************************************
'<comment>
' <summary>
'  The terminater for the class. It closes an internet session.
' </summary>
'</comment>
'****************************************************************************
Private Sub Class_Terminate()
    On Error Resume Next
    Call InternetCloseHandle(hHttpOpenRequest)
    Call InternetCloseHandle(hInternetConnect)
    Call InternetCloseHandle(hInternetSession)
End Sub
                        
'****************************************************************************
'<comment>
' <summary>
'   Executes the specified transaction against a RETS server.
'  </summary>
' <remarks></remarks>
' <param name="transaction">
'  A RETSTransaction that is to be requested from a RETS server.
' </param>
'</comment>
'****************************************************************************
Public Sub Execute(transaction As RETSTransaction)

        Set transaction.context = connectionContext
        If transaction.requestType = "Login" Then
            InitializeSession
        End If
        
        transaction.Preprocess
        Call ProcessRETSTransaction(transaction)
        transaction.Postprocess
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Gets the target url for the transaction
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get url() As String
    url = serverUrl
End Property

'****************************************************************************
'<comment>
' <summary>Sets the target url to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new url property value.
' </param>
'</comment>
'****************************************************************************

Public Property Let url(RHS As String)
    serverUrl = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the username for the session
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Property Get UserName() As String
    UserName = connectionContext.Item("username")
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the password for the session
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Property Get Password() As String
    Password = connectionContext.Item("password")
End Property

'****************************************************************************
'<comment>
' <summary>
'   Sets key-value pairs in the client request header
' </summary>
' <remarks></remarks>
' <param name="key">
'  A String that is used as the new key.
' </param>
' <param name="value">
'  A String that is used as the new value.
' </param>
'</comment>
'****************************************************************************
Public Sub SetRequestHeaderField(key As String, value As String)
    headerMap.Item(key) = value
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Removes key-value pairs in the client request header
' </summary>
' <remarks></remarks>
' <param name="key">
'  A String that is used as the new key.
' </param>
'</comment>
'****************************************************************************
Public Sub RemoveRequestHeaderField(key As String)
    headerMap.Remove key
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Based on the URL and Request parameters, GetURLContent makes a HTTP request
'       and returns the response.
' </summary>
' <remarks></remarks>
' <param name="strURL">
'  A String that contains the URL for the HTTP request.
' </param>
' <param name="requestMethod">
'  A String that contains the HTTP request Type 'GET' or 'POST'.
' </param>
' <param name="requestMap">
'  A Dictionary that contains the request parameters.
' </param>
'</comment>
'****************************************************************************
Public Function GetURLContent(strURL As String, _
                            requestMethod As String, _
                            requestMap As Dictionary) As String
    Dim contents As String
    Dim reqStr As String
    Dim iRet As Integer


    cat.debugLog "getURLContent: URL=" & strURL
    cat.debugLog "getURLContent: requestMethod=" & requestMethod
    cat.debugLog "Cookies Header: " & headerMap.Item("Cookie")
   
'    Dim username As String
'    username = GetUsername()
'    Dim password As String
'    password = GetPassword()
    
    Dim urlParts As URL_COMPONENTSA
    urlParts = CrackURL(strURL)
    
    Dim host As String
    host = urlParts.lpszHostName
    
    Dim port As Integer
    port = urlParts.nPort
    'port = INTERNET_INVALID_PORT_NUMBER
    
    Dim Path As String
    Path = urlParts.lpszUrlPath
    
    Dim scheme As Long
    scheme = INTERNET_SERVICE_HTTP
    
    Dim optSSL As Boolean
    Select Case urlParts.nScheme
        Case INTERNET_SCHEME_HTTP
            optSSL = False
        Case INTERNET_SCHEME_HTTPS
            optSSL = True
    End Select
    
    If Me.requestId = Null Or Len(Me.requestId) = 0 Then
        Me.requestId = getRequestID()
    End If
    
    SetRequestHeaderField "Accept", "*/*"
    'SetRequestHeaderField "Accept-Encoding", "gzip, deflate"

    SetRequestHeaderField "User-Agent", userAgent

    SetRequestHeaderField "RETS-Version", RETS_Version
    SetRequestHeaderField "Content-Type", "application/x-www-form-urlencoded"
    SetRequestHeaderField "RETS-Request-ID", Me.requestId
    ' setRequestHeaderField "Host", Host ' marketlinx fix?

    ' Create a HTTP Connection to the remote URL
    hInternetConnect = InternetConnect(hInternetSession, host, port, _
                    UserName, Password, INTERNET_SERVICE_HTTP, 0, 0)

    If (hInternetConnect = 0) Then
        GetURLContent = ""
        cat.errorLog (GetError())
        Exit Function
    End If

    ' build the request parameters string
    reqStr = BuildQueryString(requestMap)
    
    ' create request
    Const protocol As String = "HTTP/1.1"
    
    Dim flags As Long
    Dim sslFlags As Long
    Dim postFlags As Long
    
    flags = INTERNET_FLAG_RELOAD Or INTERNET_FLAG_KEEP_CONNECTION
    flags = flags Or INTERNET_FLAG_NO_AUTH  '   needed so we can do own digest authentication
    sslFlags = INTERNET_FLAG_SECURE Or INTERNET_FLAG_IGNORE_CERT_CN_INVALID Or INTERNET_FLAG_IGNORE_CERT_DATE_INVALID
    postFlags = INTERNET_FLAG_MULTIPART
        
    If (StrComp(requestMethod, "GET", vbTextCompare) = 0) Then
        If (Len(Trim(reqStr)) > 0) Then
            Path = Path & "?" & Trim(reqStr)
        End If
        
        If optSSL = False Then
            hHttpOpenRequest = HttpOpenRequest(hInternetConnect, "GET", Path, protocol, vbNullString, 0, _
            flags, 0)
        Else
            hHttpOpenRequest = HttpOpenRequest(hInternetConnect, "GET", Path, protocol, vbNullString, 0, _
            flags Or sslFlags, 0)
        End If
    Else 'POST
        If optSSL = False Then
            hHttpOpenRequest = HttpOpenRequest(hInternetConnect, "POST", Path, protocol, vbNullString, 0, _
            flags Or postFlags, 0)
        Else
            hHttpOpenRequest = HttpOpenRequest(hInternetConnect, "POST", Path, protocol, vbNullString, 0, _
            flags Or postFlags Or sslFlags, 0)
       End If
    End If
    
    ' Set timeouts;
    Dim dwTimeOut As Long
    dwTimeOut = 60000   ' 60 seconds

    iRet = InternetSetOption(hHttpOpenRequest, INTERNET_OPTION_CONNECT_TIMEOUT, dwTimeOut, 4)


    If (iRet = 0) Then
        cat.errorLog (GetError())
    End If

    iRet = InternetSetOption(hHttpOpenRequest, INTERNET_OPTION_RECEIVE_TIMEOUT, dwTimeOut, 4)


    If (iRet = 0) Then
        cat.errorLog (GetError())
    End If

    iRet = InternetSetOption(hHttpOpenRequest, INTERNET_OPTION_SEND_TIMEOUT, dwTimeOut, 4)

    If (iRet = 0) Then
        cat.errorLog (GetError())
    End If

resend:
        Call SetAuthorization(hHttpOpenRequest, Path, UserName, Password)
        
        ' add required headers
        AddToHttpHeader hHttpOpenRequest, headerMap
    
        ' make request
        iRet = HttpSendRequest(hHttpOpenRequest, vbNullString, 0, reqStr, Len(reqStr))

        If (iRet = 0) Then
            cat.errorLog (GetError())
            GetURLContent = ""
            Exit Function
        End If
                    
        ' Check the response code
        Dim responseCode As Integer
        responseCode = GetResponseCode(hHttpOpenRequest)
        cat.debugLog "Status Code:" & responseCode
        
        ' display all headers
        Call DisplayHeaders(hHttpOpenRequest)
        
        ' store all headers into response map
        responseMap.removeAll
        Call StoreHeadersResponseMap(hHttpOpenRequest, responseMap)
        
        ' get cookie
        Dim cookie As String
        cookie = GetCookie(hHttpOpenRequest)

        ' add cookie to header
        If cookie <> "" Then
            cat.debugLog "Setting Cookie:" + cookie
            headerMap.Item("Cookie") = cookie
        End If
        
        '// if error
        If (responseCode <> HTTP_STATUS_OK _
                And (responseCode <> HTTP_STATUS_DENIED Or _
                    (isRetryingAuthorization = True And IsStale(responseMap) <> True))) Then

            ' Collect the contents for error message
            contents = ReadContents(hHttpOpenRequest)

            errMsg = "Unable to connect to """ & strURL & """ http response code:" & responseCode & " Contents:[" & contents & "]"
            cat.errorLog (errMsg)
            contents = ""
        
        ElseIf (responseCode = HTTP_STATUS_DENIED) Then
            Call SetAuthorization(hHttpOpenRequest, Path, UserName, Password)

            isRetryingAuthorization = True
            cat.debugLog "RETRYING AUTHORIZATION!"
            GoTo resend 'Sorry for the GoTo
        Else
            contents = ReadContents(hHttpOpenRequest)
        End If
        
        isRetryingAuthorization = False
        GetURLContent = contents
End Function

'****************************************************************************
'<comment>
' <summary>
'  private method that sets up the http header for each request.
' </summary>
' <remarks></remarks>
' <param name="hHttpOpenRequest">
'  A HANDLE to the httpOpenRequest Object.
' </param>
' <param name="headers">
'  A Dictionary that contains HTTP Header value pairs.
' </param>
'</comment>
'****************************************************************************
Private Sub AddToHttpHeader(hHttpOpenRequest As Long, headers As Dictionary)
    Dim sHeaders As String
    Dim key As Variant
    Dim value As String
    Dim iRet As Integer
    
    Dim Keys() As Variant
    Keys = headers.Keys()
    For Each key In Keys
        value = headers.Item(key)
        If (IsValidString(value)) Then
            sHeaders = key & ": " & value & vbCrLf
            iRet = HttpAddRequestHeaders(hHttpOpenRequest, sHeaders, Len(sHeaders), _
                HTTP_ADDREQ_FLAG_ADD Or HTTP_ADDREQ_FLAG_REPLACE)
        End If
    Next key
End Sub

'****************************************************************************
'<comment>
' <summary>
'  private method that sends the transaction to the server and processes the response.
' </summary>
' <remarks></remarks>
' <param name="transaction">
'  A RETSTransaction to be processed.
' </param>
'</comment>
'****************************************************************************
Private Sub ProcessRETSTransaction(transaction As RETSTransaction)

    Dim httpMethod As String

    Dim contents As String
    contents = ""

    serverUrl = transaction.url

    cat.debugLog transaction.requestType & " URL : {" & serverUrl & "}"

    If (serverUrl = Null) Then
        cat.errorLog transaction.requestType & " URL is null"

        transaction.responseStatus = "20036"
        transaction.responseStatusText = transaction.requestType & " URL is missing. Successful login is required."
        Exit Sub
    End If
    
    ' set http method
    If (transaction.requestType = "Action") Then
        httpMethod = "GET"  ' RETS spec requires Action to use GET
    Else
        'httpMethod = "GET"
        httpMethod = "POST"
    End If

    contents = GetURLContent(serverUrl, httpMethod, transaction.requestMap)

    If (responseMap.Count > 0) Then
        Set transaction.responseMap = responseMap
    End If
        
    If (transaction.requestType = "GetObject") Then
        transaction.responseStatus = "0"
        transaction.responseStatusText = ""
        transaction.response = contents
    ElseIf (Len(contents) = 0) Then
        transaction.responseStatus = "20513" ' Miscellaneous error
        transaction.responseStatusText = errMsg
        transaction.response = errMsg
        errMsg = ""
    Else
        Dim firstStatusRE As New RegExp
        Dim firstMatches As MatchCollection
        Dim firstMatch As Match
        Dim secondStatusRE As New RegExp
        Dim secondMatches As MatchCollection
        Dim secondMatch As Match
        
        transaction.response = contents
        firstStatusRE.Pattern = "<RETS\s?ReplyCode=""(.*?)""\s?ReplyText=""(.*?)"""
        secondStatusRE.Pattern = "<RETS-STATUS\s?ReplyCode=""(.*?)""\s?ReplyText=""(.*?)"""

        Call cat.debugLog("looking for <RETS tag")
        
        Set firstMatches = firstStatusRE.Execute(contents)
        
        If (firstMatches.Count = 0) Then
            transaction.responseStatus = ""
            transaction.responseStatusText = _
                "The response is not a valid format for a RETS response, a " & _
                "valid (<RETS ReplyCode="""" ReplyText="""" >) tag was not found."
            Exit Sub
        End If
        
        Set firstMatch = firstMatches(0)
        transaction.responseStatus = firstMatch.SubMatches(0)
        transaction.responseStatusText = firstMatch.SubMatches(1)

        ' extract RETS-STATUS values is any
        Set secondMatches = secondStatusRE.Execute(contents)
        
        If (secondMatches.Count > 0) Then
            Set secondMatch = secondMatches(0)
            transaction.responseStatus = secondMatch.SubMatches(0)
            transaction.responseStatusText = secondMatch.SubMatches(1)
        End If
    End If
End Sub

'****************************************************************************
' NEW Function 10/04/2005 (ASO) - Changes supporting Digest Authentication.
'       See RETS1.5 specification (section 4.1.2)
'<comment>
' <summary>
'  private method that takes 4 strings as input and returns a DigestAuthentication-ized
'  string to send in the RETS request as the Cnonce value
' </summary>
' <remarks></remarks>
' <param name="userAgent">
'  UserAgent header value.
' </param>
' <param name="ClientPassword">
'  The Client Password also called the Cnonce Password (see Spec section 4.1.2)
' </param>
' <param name="requestId">
'  A "made-up" RequestID value that must be unique.  Using a Timestamp.
' </param>
' <param name="requestId">
'  The server-returned nonce string
' </param>
'</comment>
'****************************************************************************


Private Function ClientAuthentication(userAgent As String, ClientPassword As String, _
                                     requestId As String, _
                                     nonce As String) As String
                                     
    Dim clientAuthString    As String
    Dim digestAuthString    As String
    
    clientAuthString = userAgent & ":" & ClientPassword & ":" & requestId & ":" & nonce
    digestAuthString = DigestMessage(clientAuthString)
    cat.debugLog ("evaluateMD5ResponseDigest: digestAuthString(" & clientAuthString & ")=" & digestAuthString)
    
    ClientAuthentication = digestAuthString

End Function

'****************************************************************************
' NEW Function 10/05/2005 (ASO) - Changes supporting Digest Authentication.
'       See RETS1.5 specification (section 4.1.2)
'<comment>
' <summary>
'  private method that creates a Unique requestID value for the preparation of the cnonce value
'  to be sent to the server.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************

Private Function getRequestID() As String
    Dim sRequestID      As String
    Dim currTime        As Date
    Dim iPosition       As Integer
    
    currTime = Time
    sRequestID = Format(currTime, "General Number", vbUseSystemDayOfWeek, vbUseSystem) 'CStr(currTime)
    iPosition = InStr(1, sRequestID, ".")
    If iPosition > 0 Then
        sRequestID = Replace(sRequestID, ".", "")
    End If
    Debug.Print ("requestID value: " + sRequestID)
    
    getRequestID = sRequestID
End Function

'****************************************************************************
'<comment>
' <summary>
'  Create digest Authentication String
' </summary>
' <remarks></remarks>
' <param name="username">
'  The username used for authentication.
' </param>
' <param name="password">
'  The password used for authentication.
' </param>
' <param name="requestMethod">
'  The HTTP method used GET|POST.
' </param>
' <param name="uri">
'  The uri that needs authentication.
' </param>
' <param name="authenticate">
'  The value of the header WWW-Authenticate.
' </param>
'</comment>
'****************************************************************************
Private Function DigestAuthorization(UserName As String, _
         Password As String, _
         requestMethod As String, _
         uri As String, _
         authenticate As String) As String

'***
'Revisions to this method as of 10/5/05 (ASO)
'
'***

'commented out 10/04/2005 (ASO)
     'Dim m As Dictionary
     'Set m = ParseAuthenticate(Mid(authenticate, 8))
     Set authenticateDictionary = ParseAuthenticate(Mid(authenticate, 8))
     
     Dim realm      As String
     Dim nonce      As String
     Dim opaque     As String
     Dim nc         As String
     
     realm = authenticateDictionary.Item("realm")
     nonce = authenticateDictionary.Item("nonce")
     opaque = authenticateDictionary.Item("opaque")
     nc = "00000001"
     
     Dim cnonce         As String
     Dim sRequestID     As String
     
     If Me.requestId = Null Or Len(Trim(Me.requestId)) = 0 Then
        sRequestID = getRequestID()
     Else
        sRequestID = Me.requestId
     End If
     'cnonce = "0a4f113b"
     cnonce = ClientAuthentication(Me.userAgent, Me.ClientPassword, sRequestID, nonce)
     Dim qop As String
     qop = authenticateDictionary.Item("qop")

     Dim isRFC2617 As Boolean
     isRFC2617 = (qop <> "") ' Recognizes differences between RFC2069 and RFC2617

     Dim a1 As String
     Dim digestA1 As String
     Dim a2 As String
     Dim digestA2 As String
     Dim response As String
     Dim digestResponse As String
     digestResponse = ""

    a1 = UserName & ":" & realm & ":" & Password
    
    digestA1 = DigestMessage(a1)
    cat.debugLog ("evaluateMD5ResponseDigest: digestA1(" & a1 & ")=" & digestA1)
    
    a2 = requestMethod & ":" & uri
    digestA2 = DigestMessage(a2)
    cat.debugLog ("evaluateMD5ResponseDigest: digestA2(" & a2 & ")=" & digestA2)
    
    response = digestA1 & ":" & nonce
    
    If (isRFC2617) Then
        response = response & ":" & nc & ":" & cnonce & ":" & qop
    End If
    
    response = response & ":" & digestA2
    
    digestResponse = DigestMessage(response)
    cat.debugLog ("evaluateMD5ResponseDigest: digestResponse(" & response & ")=" & digestResponse)
    

     Dim digest As String
     digest = "Digest username=""" & UserName & """, " & _
             "realm=""" & realm & """, " & _
             "nonce=""" & nonce & """, " & _
             "opaque=""" & opaque & """, " & _
             "uri=""" & uri & """, " & _
             "response=""" & digestResponse & """"

     If (isRFC2617) Then
         digest = digest & ", qop=""" & qop & """, " & _
             "cnonce=""" & cnonce & """, " & _
             "nc=" & nc
     End If

     DigestAuthorization = digest

End Function

'****************************************************************************
'<comment>
' <summary>
'     Looks for stale="true" in Authorization Header parameters.
' </summary>
' <remarks></remarks>
' <param name="responseMap">
'  The header parameters.
' </param>
'</comment>
'****************************************************************************
Private Function IsStale(responseMap As Dictionary) As Boolean
    Dim digestAuth As New Dictionary
    
    IsStale = False
    
    If (responseMap.Exists("WWW-Authenticate") = True) Then
        Set digestAuth = ParseAuthenticate(responseMap.Item("WWW-Authenticate"))
    
        If (digestAuth.Exists("stale") = True) Then
            If (digestAuth.Item("stale") = "true") Then
                IsStale = True
            End If
        End If
    End If


End Function
'****************************************************************************
'<comment>
' <summary>
'     Parses the WWW-Authenticate value and returns a Dictionary.
' </summary>
' <remarks></remarks>
' <param name="s">
'  A String to be parsed.
' </param>
'</comment>
'****************************************************************************
Private Function ParseAuthenticate(s As String) As Dictionary

    Dim key As String
    key = ""
    Dim value As String
    value = ""
    Dim equalSign As Integer
    equalSign = -1
    Dim aDictionary As New Dictionary

    Dim list() As String
    list = Split(s, ",")
    
    Dim token
    
    For Each token In list
        equalSign = InStr(token, "=")
        If (equalSign >= 0) Then
            key = Trim(Mid(token, 1, equalSign - 1))
            value = RemoveQuotes(Trim(Mid(token, equalSign + 1)))
            aDictionary.Item(key) = value
            cat.debugLog ("parseAuthenticate:[" & key & "]=[" & value & "]")
        End If
    Next token

    Set ParseAuthenticate = aDictionary

End Function

'****************************************************************************
'<comment>
' <summary>
'     * Removes the quotes on a string.
' </summary>
' <remarks></remarks>
' <param name="quotedString">
'  A String to be processed.
' </param>
'</comment>
'****************************************************************************
Private Function RemoveQuotes(quotedString As String) As String

    quotedString = Trim(quotedString)
    Dim firstQuote As String
    Dim lastQuote As String
    
    firstQuote = InStr(quotedString, """")

    If (firstQuote = 1) Then
        quotedString = Right(quotedString, Len(quotedString) - 1)
    End If

    lastQuote = InStrRev(quotedString, """")
    If (lastQuote = Len(quotedString)) Then
        quotedString = Left(quotedString, Len(quotedString) - 1)
    End If
    
    RemoveQuotes = quotedString

End Function
    
'****************************************************************************
'<comment>
' <summary>
' *  Checks to make sure the string passed in is a valid string parameter
'       (not null and not zero length).
' </summary>
' <remarks></remarks>
' <param name="value">
'  A String to be processed.
' </param>
'</comment>
'****************************************************************************
Private Function IsValidString(value As String) As Boolean

    IsValidString = (Len(value) > 0)

End Function
    
'****************************************************************************
'<comment>
' <summary>
'   Reads the response from an HTTP Open Request.
' </summary>
' <remarks></remarks>
' <param name="hHttpOpenRequest">
'  A Handle from an HttpOpenRequest.
' </param>
'</comment>
'****************************************************************************
Private Function ReadContents(hHttpOpenRequest As Long) As String
    Dim sBuffer As String
    Dim sReadBuffer As String * 4096
    Dim lBufferLength As Long
    Dim sNewData As String
    
    Dim bDoLoop As Boolean
    bDoLoop = True

'    While bDoLoop
'        sReadBuffer = vbNullString
'        bDoLoop = InternetReadFile(hHttpOpenRequest, sReadBuffer, Len(sReadBuffer), lBufferLength)
'
'        sNewData = Left$(sReadBuffer, lBufferLength)
'        Debug.Print lBufferLength & ":" & sNewData
'
'        sBuffer = sBuffer & sNewData
'        If (lBufferLength = 0) Then bDoLoop = False
'    Wend
    
    While bDoLoop
        sReadBuffer = vbNullString
        bDoLoop = InternetReadFile(hHttpOpenRequest, sReadBuffer, Len(sReadBuffer), lBufferLength)

        sNewData = Left$(sReadBuffer, lBufferLength)
        Debug.Print lBufferLength & ":" & sNewData

        sBuffer = sBuffer & sNewData
        If Not CBool(lBufferLength) Then bDoLoop = False
    Wend
    
'    If (responseMap.Item("Content-Encoding") = "gzip") Then
'        ReadContents = sBuffer
'    Else
        ReadContents = sBuffer
'    End If
    
End Function

'****************************************************************************
'<comment>
' <summary>
'   Build a query string based on the contents of the requestMap.
' </summary>
' <remarks></remarks>
' <param name="requestMap">
'  A Dictionary containing key-values to be put into query string.
' </param>
'</comment>
'****************************************************************************
Private Function BuildQueryString(requestMap As Dictionary) As String
        Dim reqStr As String
        reqStr = ""

    If (requestMap.Count > 0) Then
        cat.debugLog "requestMap.Count=" & requestMap.Count

        ' build query string
        Dim key As Variant
        Dim Keys() As Variant
        Dim keysIndex As Integer
        Dim keysLength As Integer
        
        Keys = requestMap.Keys()
        keysLength = UBound(Keys)
        keysIndex = LBound(Keys)
        
        For Each key In Keys
            If (Len(Trim(requestMap.Item(key))) > 0) Then
                ' url encode the value string
                reqStr = reqStr & URLEncode(Trim(key)) & "=" & URLEncode(Trim(requestMap.Item(key)))
                
                If keysIndex < keysLength Then
                    reqStr = reqStr & "&"
                End If
            End If
                
            keysIndex = keysIndex + 1
        Next key
    End If
    
    BuildQueryString = reqStr
End Function

'****************************************************************************
'<comment>
' <summary>
'   Display the Response headers thru the Logger.
' </summary>
' <remarks></remarks>
' <param name="hHttpOpenRequest">
'  A Handle to the HttpOpenRequest.
' </param>
'</comment>
'****************************************************************************
Private Function DisplayHeaders(hHttpOpenRequest As Long) As Integer

    Dim iRet As Integer
    Dim sBuffer As String * 4096
    Dim lBufferLength As Long
    
    lBufferLength = Len(sBuffer)
    iRet = HttpQueryInfo(hHttpOpenRequest, HTTP_QUERY_RAW_HEADERS_CRLF, _
             ByVal sBuffer, lBufferLength, 0)
    If (iRet = 0) Then
        cat.errorLog (GetError())
    End If
             
    cat.debugLog iRet & ":[" & lBufferLength & "] " & "HTTP_QUERY_RAW_HEADERS_CRLF:" & sBuffer
    
    DisplayHeaders = iRet
End Function

'****************************************************************************
'<comment>
' <summary>
'   Copy the Response headers into a Dictionary Object.
' </summary>
' <remarks></remarks>
' <param name="hHttpOpenRequest">
'  A Handle to the HttpOpenRequest.
' </param>
' <param name="responseMap">
'  A Dictionary the will contain key-values from the response headers.
' </param>
'</comment>
'****************************************************************************
Private Sub StoreHeadersResponseMap(hHttpOpenRequest, responseMap)

    Dim iRet As Integer
    Dim sBuffer As String * 4096
    Dim lBufferLength As Long
    Dim lNewLinePos As Long
    Dim lPrevNewLinePos As Long
    Dim lSeperatorPos As Long
    Dim sKey As String
    Dim sItem As String
    
    lBufferLength = Len(sBuffer)
    iRet = HttpQueryInfo(hHttpOpenRequest, HTTP_QUERY_RAW_HEADERS_CRLF, _
             ByVal sBuffer, lBufferLength, 0)
    
    If (iRet <> 0) Then
        lNewLinePos = InStr(1, sBuffer, vbCrLf, vbBinaryCompare)
        lPrevNewLinePos = 0
        lSeperatorPos = InStr(1, sBuffer, ": ", vbBinaryCompare)
        
        Do While ((lNewLinePos > 0) Or (lSeperatorPos > 0))
            If ((lSeperatorPos > 0) And _
                ((lNewLinePos = 0) Or (lNewLinePos > lSeperatorPos))) Then
                If ((lSeperatorPos > lPrevNewLinePos + 2) And _
                    (lNewLinePos > lSeperatorPos + 2)) Then
                    sKey = _
                        Trim(Mid(sBuffer, lPrevNewLinePos + 2, lSeperatorPos - _
                            (lPrevNewLinePos + 2)))
                    sItem = _
                        Trim(Mid(sBuffer, lSeperatorPos + 2, lNewLinePos - _
                            (lSeperatorPos + 2)))
                    responseMap.Item(sKey) = sItem
                End If
                
                lPrevNewLinePos = lNewLinePos
                lNewLinePos = InStr(lNewLinePos + 1, sBuffer, vbCrLf, vbBinaryCompare)
                lSeperatorPos = InStr(lSeperatorPos + 1, sBuffer, ": ", vbBinaryCompare)
            ElseIf (lSeperatorPos = 0) Then
                lNewLinePos = 0
            ElseIf (lNewLinePos > 0) Then
                lPrevNewLinePos = lNewLinePos
                lNewLinePos = InStr(lNewLinePos + 1, sBuffer, vbCrLf, vbBinaryCompare)
            End If
        Loop
    End If
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Returns a Cookie from the HTTP response.
' </summary>
' <remarks></remarks>
' <param name="hHttpOpenRequest">
'  A Handle to the HttpOpenRequest.
' </param>
'</comment>
'****************************************************************************
Private Function GetCookie(hHttpOpenRequest As Long) As String
    Dim iRet As Integer
    Dim sBuffer As String * 4096
    Dim lBufferLength As Long
    
    lBufferLength = Len(sBuffer)
    iRet = HttpQueryInfo(hHttpOpenRequest, HTTP_QUERY_SET_COOKIE, _
             ByVal sBuffer, lBufferLength, 0)
    If (iRet = 0) Then
        cat.errorLog (GetError())
        GetCookie = ""
    Else
        GetCookie = Left(sBuffer, InStr(sBuffer, ";"))
    End If
End Function

'****************************************************************************
'<comment>
' <summary>
'   Sets up the username and password for the HTTP Request.
' </summary>
' <remarks></remarks>
' <param name="hHttpOpenRequest">
'  A Handle to the HttpOpenRequest.
' </param>
' <param name="username">
'  A String containing the username.
' </param>
' <param name="password">
'  A String containing the password.
' </param>
'</comment>
'****************************************************************************
Private Sub SetAuthorization(hHttpOpenRequest As Long, Path As String, UserName As String, Password As String)

    Dim iRet As Integer
    Dim sBuffer As String * 1024
    Dim lBufferLength As Long
    

    iRet = InternetSetOptionStr(hHttpOpenRequest, INTERNET_OPTION_USERNAME, _
                      UserName, Len(UserName) + 1)
    If (iRet = 0) Then
        cat.errorLog (GetError())
    End If
    
    iRet = InternetSetOptionStr(hHttpOpenRequest, INTERNET_OPTION_PASSWORD, _
                      Password, Len(Password) + 1)
    If (iRet = 0) Then
        cat.errorLog (GetError())
    End If
                    
    ' get WWW-Authenticate
    lBufferLength = Len(sBuffer)
    iRet = HttpQueryInfo(hHttpOpenRequest, HTTP_QUERY_WWW_AUTHENTICATE, _
             ByVal sBuffer, lBufferLength, 0)
    If (iRet = 1) Then
        wwwAuthenticate = Left(sBuffer, lBufferLength)
    End If
        
    ' if starts with "Digest "
    If (InStr(wwwAuthenticate, "Digest ") = 1) Then ' Digest Authentication
        cat.debugLog "Digest Auth : " + wwwAuthenticate
        'get requestMethod
        Dim sRequestMethod As String
        lBufferLength = Len(sBuffer)
        iRet = HttpQueryInfo(hHttpOpenRequest, HTTP_QUERY_REQUEST_METHOD, _
             ByVal sBuffer, lBufferLength, 0)
        If (iRet <> 1) Then
            cat.errorLog (GetError())
        Else
            sRequestMethod = Left(sBuffer, lBufferLength)
            'get digest
            Dim digest As String
            digest = DigestAuthorization(UserName, Password, _
                                sRequestMethod, Path, wwwAuthenticate)
            cat.debugLog "Digest response Str : " & digest
            headerMap.Item("Authorization") = digest
        End If
    ElseIf (InStr(wwwAuthenticate, "Basic ") = 1) Then ' Basic Authentication
        Dim authStr As String
        Dim aBase64 As New base64
        
        authStr = UserName & ":" & Password
        cat.debugLog "Basic Auth Str : " & authStr
        headerMap.Item("Authorization") = "Basic " & aBase64.Encode(authStr)
    Else
        If headerMap.Exists("Authorization") Then
            headerMap.Remove ("Authorization") ' clear authorization
        End If
    End If
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Retrieves the http response code for the HTTP Request.
' </summary>
' <remarks></remarks>
' <param name="hHttpOpenRequest">
'  A Handle to the HttpOpenRequest.
' </param>
'</comment>
'****************************************************************************
Private Function GetResponseCode(hHttpOpenRequest) As Integer
    Dim iRet As Integer
    Dim sBuffer As String * 4096
    Dim lBufferLength As Long
    
    Dim responseCode As Integer
    responseCode = -1
    
    lBufferLength = Len(sBuffer)
    iRet = HttpQueryInfo(hHttpOpenRequest, _
            HTTP_QUERY_STATUS_CODE, _
            ByVal sBuffer, lBufferLength, 0)
    If (iRet = 0) Then
        cat.errorLog (GetError())
    Else
        responseCode = CInt(Left(sBuffer, lBufferLength))
    End If
    
    GetResponseCode = responseCode
End Function


'****************************************************************************
'<comment>
' <summary>
'   Returns the MD5 hash of inString.
' </summary>
' <remarks></remarks>
' <param name="inString">
'  string to be MD5 hashed.
' </param>
'</comment>
'****************************************************************************
Function DigestMessage(InString As String) As String
    Dim md As New MD5
    
    md.Text = InString
    
    DigestMessage = md.HEXMD5
End Function

