VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RETSGetMetadataTransaction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************************************
'<comment>
' <summary>
'  RETSGetMetadataTransaction class encapsulates the RETS GetMetadata transaction
'  It implements RETSTransaction interface.
'   See the RETSTransaction class for 'inherited' details.
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
Option Explicit
Implements RETSTransaction

'****************************************************************************
'<comment>
' <summary>
'  The private m_transaction object to which transactions methods and attributes are delegated.
' </summary>
'</comment>
'****************************************************************************
Private m_transaction As New RETSTransaction


'****************************************************************************
'<comment>
' <summary>
'  The initializer for the class. It initializes the requestType of the transaction.
' </summary>
'</comment>
'****************************************************************************
Private Sub Class_Initialize()
    m_transaction.requestType = "GetMetadata"
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''
'   setter/getters methods
'''''''''''''''''''''''''''''''''''''''''''''''''''''

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction property
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get transaction() As RETSTransaction
    Set transaction = m_transaction
End Property

'****************************************************************************
'<comment>
' <summary>
'   Sets the transaction metadataType property to a new value.
'   Sets up both "Type" and "type" to be sent as the request parameters.
'  </summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new metadataType property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let metadataType(RHS As String)
    m_transaction.requestMap.Item("Type") = RHS
    'm_transaction.requestMap.Item("type") = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Sets the transaction metaDataID property to a new value.
'   Sets up both "ID" and "id" to be sent as the request parameters.
' </summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new metaDataID property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let metaDataID(RHS As String)
    m_transaction.requestMap.Item("ID") = RHS
    'm_transaction.requestMap.Item("id") = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Sets the transaction metadataFormat property to a new value.
'   Sets up both "Format" and "format" to be sent as the request parameters.
' </summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new metadataFormat property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let metadataFormat(RHS As String)
    m_transaction.requestMap.Item("Format") = RHS
    'm_transaction.requestMap.Item("format") = RHS
End Property

'''''''''''''''''''''''''''''''''''''''''''''''''''''
'   'inherited' methods
'''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Property Get RETSTransaction_capabilityUrls() As Dictionary
    Set RETSTransaction_capabilityUrls = m_transaction.capabilityUrls
End Property

Private Property Set RETSTransaction_capabilityUrls(RHS As Dictionary)
    Set m_transaction.capabilityUrls = RHS
End Property

Private Property Get RETSTransaction_context() As Dictionary
    Set RETSTransaction_context = m_transaction.context
End Property

Private Property Set RETSTransaction_context(aContext As Dictionary)
    Set m_transaction.context = aContext
End Property

Private Property Get RETSTransaction_requestMap() As Dictionary
    Set RETSTransaction_requestMap = m_transaction.requestMap
End Property

Private Property Let RETSTransaction_requestType(RHS As String)
    m_transaction.requestType = RHS
End Property

Private Property Get RETSTransaction_requestType() As String
    RETSTransaction_requestType = m_transaction.requestType
End Property

Private Property Get RETSTransaction_response() As String
    RETSTransaction_response = m_transaction.response
End Property

Private Property Let RETSTransaction_response(RHS As String)
    m_transaction.response = RHS
End Property

Private Property Get RETSTransaction_responseMap() As Dictionary
    Set RETSTransaction_responseMap = m_transaction.responseMap
End Property

Private Property Set RETSTransaction_responseMap(aResponseMap As Dictionary)
    Set m_transaction.responseMap = aResponseMap
End Property

Private Property Let RETSTransaction_responseStatus(RHS As String)
    m_transaction.responseStatus = RHS
End Property

Private Property Get RETSTransaction_responseStatus() As String
    RETSTransaction_responseStatus = m_transaction.responseStatus
End Property

Private Property Get RETSTransaction_responseStatusText() As String
    RETSTransaction_responseStatusText = m_transaction.responseStatusText
End Property

Private Property Let RETSTransaction_responseStatusText(RHS As String)
    m_transaction.responseStatusText = RHS
End Property

Private Sub RETSTransaction_setKeyValuePairs(str As String)
    m_transaction.setKeyValuePairs (str)
End Sub

Private Property Get RETSTransaction_url() As String
    RETSTransaction_url = m_transaction.url
End Property

Private Property Let RETSTransaction_url(aUrl As String)
    m_transaction.url = aUrl
End Property

Private Sub RETSTransaction_Preprocess()
    Call m_transaction.Preprocess
End Sub

Private Sub RETSTransaction_Postprocess()
    Call m_transaction.Postprocess
End Sub




