VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RETSGetObjectTransaction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************************************
'<comment>
' <summary>
'  RETSGetObjectTransaction class encapsulates the RETS GetObject transaction
'  It implements RETSTransaction interface.
'   See the RETSTransaction class for 'inherited' details.
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
'****************************************************************************
'<comment>
' <summary>
'  RETSGetObjectTransaction class encapsulates the RETS GetObject transaction
'  It implements RETSTransaction interface.
'   See the RETSTransaction class for 'inherited' details.
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
Option Explicit
Implements RETSTransaction


' HTTP Header Keys
Private Const CONTENT_TYPE_KEY = "Content-Type"
Private Const LOCATION_KEY = "Location"

'****************************************************************************
'<comment>
' <summary>
'  The private m_transaction object to which transactions methods and attributes are delegated.
' </summary>
'</comment>
'****************************************************************************
Private m_transaction As New RETSTransaction

'   Required Arguments
Private Const C_RESOURCE = "Resource"
Private Const C_TYPE = "Type"
Private Const C_ID = "ID"

'   Optional Arguments
Private Const C_LOCATION = "Location"


'****************************************************************************
'<comment>
' <summary>
'  The initializer for the class. It initializes the requestType of the transaction.
' </summary>
'</comment>
'****************************************************************************
Private Sub Class_Initialize()
    m_transaction.requestType = "GetObject"
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction property
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get transaction() As RETSTransaction
    Set transaction = m_transaction
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction contentType property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get contentType() As String
    contentType = m_transaction.responseMap.Item(CONTENT_TYPE_KEY)
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction URI location property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get Location() As String
    Location = m_transaction.responseMap.Item(LOCATION_KEY)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Resource property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Resource property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let objectResource(RHS As String)
    m_transaction.requestMap.Item(C_RESOURCE) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction Resource property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get objectResource() As String
    objectResource = m_transaction.requestMap.Item(C_RESOURCE)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Type property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Type property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let objectType(RHS As String)
    m_transaction.requestMap.Item(C_TYPE) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction Type property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get objectType() As String
    objectType = m_transaction.requestMap.Item(C_TYPE)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction ID property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new ID property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let objectID(RHS As String)
    m_transaction.requestMap.Item(C_ID) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction ID property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get objectID() As String
    objectID = m_transaction.requestMap.Item(C_ID)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Location flag property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Location flag property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let objectLocation(RHS As String)
    m_transaction.requestMap.Item(C_LOCATION) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction Location flag property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get objectLocation() As String
    objectLocation = m_transaction.requestMap.Item(C_LOCATION)
End Property

'''''''''''''''''''''''''''''''''''''''''''''''''''''
'   'inherited' methods
'''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Property Get RETSTransaction_capabilityUrls() As Dictionary
    Set RETSTransaction_capabilityUrls = m_transaction.capabilityUrls
End Property

Private Property Set RETSTransaction_capabilityUrls(RHS As Dictionary)
    Set m_transaction.capabilityUrls = RHS
End Property

Private Property Get RETSTransaction_context() As Dictionary
    Set RETSTransaction_context = m_transaction.context
End Property

Private Property Set RETSTransaction_context(aContext As Dictionary)
    Set m_transaction.context = aContext
End Property

Private Property Get RETSTransaction_requestMap() As Dictionary
    Set RETSTransaction_requestMap = m_transaction.requestMap
End Property

Private Property Let RETSTransaction_requestType(RHS As String)
    m_transaction.requestType = RHS
End Property

Private Property Get RETSTransaction_requestType() As String
    RETSTransaction_requestType = m_transaction.requestType
End Property

Private Property Get RETSTransaction_response() As String
    RETSTransaction_response = m_transaction.response
End Property

Private Property Let RETSTransaction_response(RHS As String)
    m_transaction.response = RHS
End Property

Private Property Get RETSTransaction_responseMap() As Dictionary
    Set RETSTransaction_responseMap = m_transaction.responseMap
End Property

Private Property Set RETSTransaction_responseMap(aResponseMap As Dictionary)
    Set m_transaction.responseMap = aResponseMap
End Property

Private Property Let RETSTransaction_responseStatus(RHS As String)
    m_transaction.responseStatus = RHS
End Property

Private Property Get RETSTransaction_responseStatus() As String
    RETSTransaction_responseStatus = m_transaction.responseStatus
End Property

Private Property Get RETSTransaction_responseStatusText() As String
    RETSTransaction_responseStatusText = m_transaction.responseStatusText
End Property

Private Property Let RETSTransaction_responseStatusText(RHS As String)
    m_transaction.responseStatusText = RHS
End Property

Private Sub RETSTransaction_setKeyValuePairs(str As String)
    m_transaction.setKeyValuePairs (str)
End Sub

Private Property Get RETSTransaction_url() As String
    RETSTransaction_url = m_transaction.url
End Property

Private Property Let RETSTransaction_url(aUrl As String)
    m_transaction.url = aUrl
End Property

Private Sub RETSTransaction_Preprocess()
    Call m_transaction.Preprocess
End Sub

Private Sub RETSTransaction_Postprocess()
    Call m_transaction.Postprocess
End Sub


