VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RETSLoginTransaction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************************************
'<comment>
' <summary>
'  RETSLoginTransaction class encapsulates the RETS Login transaction
'  It implements RETSTransaction interface.
'   See the RETSTransaction class for 'inherited' details.
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
Option Explicit
Implements RETSTransaction

Private Const METADATA_VERSION_KEY = "MetadataVersion"

Private m_capNameList   '   an array of valid capability names
Private m_UserName As String
Private m_password As String

'****************************************************************************
'<comment>
' <summary>
'  The private m_transaction object to which transactions methods and attributes are delegated.
' </summary>
'</comment>
'****************************************************************************
Private m_transaction As New RETSTransaction

'****************************************************************************
'<comment>
' <summary>
'  The initializer for the class. It initializes the capabilityNameList and
'       the requestType of the transaction.
' </summary>
'</comment>
'****************************************************************************
Private Sub Class_Initialize()

    m_capNameList = Array( _
            "Login", _
            "Action", _
            "ChangePassword", _
            "GetObject", _
            "LoginComplete", _
            "Logout", _
            "Search", _
            "GetMetadata", _
            "Update" _
            )
            
    m_transaction.requestType = "Login"


End Sub

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction property
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get transaction() As RETSTransaction
    Set transaction = m_transaction
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction metadataVersion property from the response.
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get MetadataVersion() As String
    MetadataVersion = m_transaction.responseMap.Item(METADATA_VERSION_KEY)
End Property


'****************************************************************************
'<comment>
' <summary>
'   Sets the transaction username property to a new value.
'  </summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new metadataType property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let UserName(RHS As String)
    m_UserName = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Sets the transaction password property to a new value.
'  </summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new metadataType property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let Password(RHS As String)
    m_password = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Sets the transaction brokerCode property to a new value.
'  </summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new metadataType property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let BrokerCode(RHS As String)
    m_transaction.requestMap.Item("BrokerCode") = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Parses the response for the capability URLs and places fully qualified
'       URLs into RETSTransaction's capabilityUrls Dictionary
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Sub setCapabilityUrls()
    Dim entry
    Dim key
    Dim capUrl As String
    Dim qualifiedUrl As String
    Dim urlRoot As String
    
    urlRoot = getUrlRoot(RETSTransaction_url)
    
    Set RETSTransaction_capabilityUrls = New Dictionary
    
    For Each entry In m_capNameList
        For Each key In RETSTransaction_responseMap.Keys()
            If (UCase(entry) = UCase(key)) Then
                capUrl = RETSTransaction_responseMap.Item(key)
                qualifiedUrl = qualifyUrl(capUrl, urlRoot)
                RETSTransaction_capabilityUrls.Item(entry) = qualifiedUrl
                If (entry = "Login") Then   ' login may reset rootUrl
                    urlRoot = getUrlRoot(qualifiedUrl)
                End If
            End If
        Next key
    Next entry
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Makes sure url is fully qualified.
'  </summary>
' <remarks></remarks>
' <param name="aUrl">
'  A String that contains a URL which may not be fully qualified.
' </param>
' <param name="defaultRoot">
'  A String that contains a root URL which may be prepended to 'aUrl'.
' </param>
'</comment>
'****************************************************************************
Private Function qualifyUrl(aUrl As String, defaultRoot As String) As String
    Dim root As String
    Dim sep As String
    
    root = getUrlRoot(aUrl)
    sep = ""
    
    If (root = Null Or root = "") Then
        If (Left(aUrl, 1) <> "/") Then
            sep = "/"
        End If
        
        qualifyUrl = defaultRoot + sep + aUrl
    Else
        qualifyUrl = aUrl
    End If
End Function


'****************************************************************************
'<comment>
' <summary>
'   Tries to extract a root from aUrl.
'  </summary>
' <remarks></remarks>
' <param name="aUrl">
'  A String that contains a URL which may not be fully qualified.
' </param>
'</comment>
'****************************************************************************
Private Function getUrlRoot(aUrl As String) As String
    Dim components As URL_COMPONENTSA
    components = CrackURL(aUrl)
    
    With components
        If (.lpszHostName <> "") Then
            getUrlRoot = .lpszScheme & "://" & .lpszHostName
        
            If .nPort <> 0 Then
                getUrlRoot = getUrlRoot & ":" & .nPort
            End If
        Else
            getUrlRoot = ""
        End If
    End With
End Function

''''''''''''''''''''''''''''''''''''''''''''
'   inherited methods
''''''''''''''''''''''''''''''''''''''''''''

Private Property Get RETSTransaction_capabilityUrls() As Dictionary
    Set RETSTransaction_capabilityUrls = m_transaction.capabilityUrls
End Property

Private Property Set RETSTransaction_capabilityUrls(RHS As Dictionary)
    Set m_transaction.capabilityUrls = RHS
End Property

Private Property Get RETSTransaction_context() As Dictionary
    Set RETSTransaction_context = m_transaction.context
End Property

Private Property Set RETSTransaction_context(aContext As Dictionary)
    Set m_transaction.context = aContext
End Property

Private Property Get RETSTransaction_requestMap() As Dictionary
    Set RETSTransaction_requestMap = m_transaction.requestMap
End Property

Private Property Let RETSTransaction_requestType(RHS As String)
    m_transaction.requestType = RHS
End Property

Private Property Get RETSTransaction_requestType() As String
    RETSTransaction_requestType = m_transaction.requestType
End Property

Private Property Get RETSTransaction_response() As String
    RETSTransaction_response = m_transaction.response
End Property

Private Property Let RETSTransaction_response(RHS As String)
    m_transaction.response = RHS
    
    Call m_transaction.setKeyValuePairs(RHS)
    Call setCapabilityUrls

End Property

Private Property Get RETSTransaction_responseMap() As Dictionary
    Set RETSTransaction_responseMap = m_transaction.responseMap
End Property

Private Property Set RETSTransaction_responseMap(aResponseMap As Dictionary)
    Set m_transaction.responseMap = aResponseMap
End Property

Private Property Let RETSTransaction_responseStatus(RHS As String)
    m_transaction.responseStatus = RHS
End Property

Private Property Get RETSTransaction_responseStatus() As String
    RETSTransaction_responseStatus = m_transaction.responseStatus
End Property

Private Property Get RETSTransaction_responseStatusText() As String
    RETSTransaction_responseStatusText = m_transaction.responseStatusText
End Property

Private Property Let RETSTransaction_responseStatusText(RHS As String)
    m_transaction.responseStatusText = RHS
End Property

Private Sub RETSTransaction_setKeyValuePairs(str As String)
    m_transaction.setKeyValuePairs (str)
End Sub

Private Property Get RETSTransaction_url() As String
    RETSTransaction_url = m_transaction.url
End Property

Private Property Let RETSTransaction_url(aUrl As String)
    m_transaction.url = aUrl
End Property

Private Sub RETSTransaction_Preprocess()
    Call m_transaction.Preprocess
    m_transaction.context.Item("username") = m_UserName
    m_transaction.context.Item("password") = m_password
End Sub

Private Sub RETSTransaction_Postprocess()
    Call m_transaction.Postprocess
End Sub

