VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RETSSearchTransaction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************************************
'<comment>
' <summary>
'  RETSSearchTransaction class encapsulates the RETS search transaction super class.
'   All other search transactions delegate to this class common RETS Search Transaction
'   methods and attributes.
'  It implements RETSTransaction interface.
'   See the RETSTransaction class for 'inherited' details.
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
Option Explicit
Implements RETSTransaction

'****************************************************************************
'<comment>
' <summary>
'   The following constants are the Dictionary Keys into the
'   request and response maps
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
    '   Required Arguments
    Private Const C_SEARCHTYPE = "SearchType"
    Private Const C_SEARCHCLASS = "Class"
    Private Const C_SEARCHQUERY = "Query"
    Private Const C_SEARCHQUERYTYPE = "QueryType"

    '   Optional Arguments
    Private Const C_SEARCHCOUNT = "Count"
    Private Const C_SEARCHFORMAT = "Format"
    Private Const C_SEARCHLIMIT = "Limit"
    Private Const C_SEARCHOFFSET = "Offset"
    Private Const C_SEARCHSELECT = "Select"
    Private Const C_SEARCHDELIMITER = "Delimiter"
    Private Const C_SEARCHRESTRICTEDINDICATOR = "RestrictedIndicator"
    Private Const C_SEARCHSTANDARDNAMES = "StandardNames"

'****************************************************************************
'<comment>
' <summary>
'  The private m_transaction object to which transactions methods and attributes are delegated to.
' </summary>
'</comment>
'****************************************************************************
    Private m_transaction As New RETSTransaction
    
    Private m_version As String

'****************************************************************************
'<comment>
' <summary>
'  The initializer for the class. It initializes the requestType of the transaction to "Search".
'   And sets the default searchQueryType to "DMQL".
' </summary>
'</comment>
'****************************************************************************
Private Sub Class_Initialize()
    m_transaction.requestType = "Search"
    
    searchQueryType = "DMQL"    '   only valid value in RETS/1.0

End Sub

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction property
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get transaction() As RETSTransaction
    Set transaction = m_transaction
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction property
'  </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get SearchTransaction() As RETSSearchTransaction
    Set SearchTransaction = Me
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction url property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new url property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let SearchType(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHTYPE) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction searchType property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get SearchType() As String
    SearchType = m_transaction.requestMap.Item(C_SEARCHTYPE)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Class property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Class property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let SearchClass(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHCLASS) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Class property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get SearchClass() As String
    SearchClass = m_transaction.requestMap.Item(C_SEARCHCLASS)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Query property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Query property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchQuery(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHQUERY) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Query property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchQuery() As String
    searchQuery = m_transaction.requestMap.Item(C_SEARCHQUERY)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction QueryType property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new QueryType property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchQueryType(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHQUERYTYPE) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction QueryType property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchQueryType() As String
    searchQueryType = m_transaction.requestMap.Item(C_SEARCHQUERYTYPE)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Count property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Count property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchCount(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHCOUNT) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Count property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchCount() As String
    searchCount = m_transaction.requestMap.Item(C_SEARCHCOUNT)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Format property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Format property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchFormat(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHFORMAT) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Format property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchFormat() As String
    searchFormat = m_transaction.requestMap.Item(C_SEARCHFORMAT)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Limit property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Limit property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchLimit(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHLIMIT) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Limit property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchLimit() As String
    searchLimit = m_transaction.requestMap.Item(C_SEARCHLIMIT)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Offset property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Offset property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchOffset(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHOFFSET) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Offset property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchOffset() As String
    searchOffset = m_transaction.requestMap.Item(C_SEARCHOFFSET)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Select property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Select property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchSelect(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHSELECT) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Select property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchSelect() As String
    searchSelect = m_transaction.requestMap.Item(C_SEARCHSELECT)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction Delimiter property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new Delimiter property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let searchDelimiter(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHDELIMITER) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction Delimiter property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get searchDelimiter() As String
    searchDelimiter = m_transaction.requestMap.Item(C_SEARCHDELIMITER)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction RestrictedIndicator property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new RestrictedIndicator property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let SearchRestrictedIndicator(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHRESTRICTEDINDICATOR) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction RestrictedIndicator property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get SearchRestrictedIndicator() As String
    SearchRestrictedIndicator = m_transaction.requestMap.Item(C_SEARCHRESTRICTEDINDICATOR)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction StandardNames property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new StandardNames property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let SearchStandardNames(RHS As String)
    m_transaction.requestMap.Item(C_SEARCHSTANDARDNAMES) = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction StandardNames property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get SearchStandardNames() As String
    SearchStandardNames = m_transaction.requestMap.Item(C_SEARCHSTANDARDNAMES)
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction version property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new version property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let version(RHS As String)
    m_version = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Gets the search transaction version property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get version() As String
    version = m_version
End Property

'''''''''''''''''''''''''''''''''''''''''''''''''''''
'   'inherited' methods
'''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Property Get RETSTransaction_capabilityUrls() As Dictionary
    Set RETSTransaction_capabilityUrls = m_transaction.capabilityUrls
End Property

Private Property Set RETSTransaction_capabilityUrls(RHS As Dictionary)
    Set m_transaction.capabilityUrls = RHS
End Property

Private Property Get RETSTransaction_context() As Dictionary
    Set RETSTransaction_context = m_transaction.context
End Property

Private Property Set RETSTransaction_context(aContext As Dictionary)
    Set m_transaction.context = aContext
End Property

Private Property Get RETSTransaction_requestMap() As Dictionary
    Set RETSTransaction_requestMap = m_transaction.requestMap
End Property

Private Property Let RETSTransaction_requestType(RHS As String)
    m_transaction.requestType = RHS
End Property

Private Property Get RETSTransaction_requestType() As String
    RETSTransaction_requestType = m_transaction.requestType
End Property

Private Property Get RETSTransaction_response() As String
    RETSTransaction_response = m_transaction.response
End Property

Private Property Let RETSTransaction_response(RHS As String)
    Dim Format As String
    
    m_transaction.response = RHS
    
    Format = UCase(m_transaction.requestMap.Item(searchFormat))
    
    If ("COMPACT" = Format Or "COMPACT-DECODED" = Format) Then
        Call processCompact(RHS)
    ElseIf ("STANDARD-XML" = Format) Then
        Call processXML(RHS)
    End If
 End Property

Private Property Get RETSTransaction_responseMap() As Dictionary
    Set RETSTransaction_responseMap = m_transaction.responseMap
End Property

Private Property Set RETSTransaction_responseMap(aResponseMap As Dictionary)
    Set m_transaction.responseMap = aResponseMap
End Property

Private Property Let RETSTransaction_responseStatus(RHS As String)
    m_transaction.responseStatus = RHS
End Property

Private Property Get RETSTransaction_responseStatus() As String
    RETSTransaction_responseStatus = m_transaction.responseStatus
End Property

Private Property Get RETSTransaction_responseStatusText() As String
    RETSTransaction_responseStatusText = m_transaction.responseStatusText
End Property

Private Property Let RETSTransaction_responseStatusText(RHS As String)
    m_transaction.responseStatusText = RHS
End Property

Private Sub RETSTransaction_setKeyValuePairs(str As String)
    m_transaction.setKeyValuePairs (str)
End Sub

Private Property Get RETSTransaction_url() As String
    RETSTransaction_url = m_transaction.url
End Property

Private Property Let RETSTransaction_url(aUrl As String)
    m_transaction.url = aUrl
End Property

Private Sub RETSTransaction_Preprocess()
    Call m_transaction.Preprocess
End Sub

Private Sub RETSTransaction_Postprocess()
    Call m_transaction.Postprocess
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'   internal methods
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Sub processCompact(body As String)

        Call processCountTag(body)
        Call processDelimiterTag(body)
        Call processColumnTag(body)
        Call processCompactData(body)
        Call processMaxRowTag(body)
End Sub

Private Sub processXML(body As String)
        Call processCountTag(body)
        Call processXMLData(body)
        Call processMaxRowTag(body)
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'   Currently these subroutines do not parse the response body.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private Sub processCountTag(body As String)
End Sub
Private Sub processDelimiterTag(body As String)
End Sub
Private Sub processColumnTag(body As String)
End Sub
Private Sub processCompactData(body As String)
End Sub
Private Sub processXMLData(body As String)
End Sub
Private Sub processMaxRowTag(body As String)
End Sub

 


