VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RETSTransaction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************************************
'<comment>
' <summary>
'  RETSTransaction class encapsulates the RETS transaction super class.
'   All other transactions delegate to this class common RETS Transaction
'   methods and attributes.
' </summary>
' <remarks></remarks>
' <author>Jeff Brush - Avantia, Inc.</author>
' <date>2002-11-06</date>
'</comment>
'****************************************************************************
Option Explicit

'****************************************************************************
'<comment>
' <summary>
'   The following constants are the Dictionary Keys into the
'   request and response maps
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private Const C_STATUS As String = "status"
Private Const C_STATUSTEXT As String = "statusText"
Private Const C_BODY As String = "body"
Private Const C_REQUESTTYPE As String = "requestType"
Private Const C_CAPABILITYURLS As String = "capabilityUrls"

'****************************************************************************
'<comment>
' <summary>
'   Holds the context information for the transaction
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_context As New Dictionary
'****************************************************************************
'<comment>
' <summary>
'   Holds the target URL for the transaction
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_url As String
'****************************************************************************
'<comment>
' <summary>
'   Holds the requestType of the transaction
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_requestType As String
'****************************************************************************
'<comment>
' <summary>
'   Holds the status for the transaction response
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_responseStatus As String
'****************************************************************************
'<comment>
' <summary>
'   Holds the status text for the transaction response
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_responseStatusText As String
'****************************************************************************
'<comment>
' <summary>
'   Holds the text of the transaction response
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_response As String

'****************************************************************************
'<comment>
' <summary>
'   Holds the request parameters for the transaction
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_requestMap As New Dictionary
'****************************************************************************
'<comment>
' <summary>
'   Holds the request parameters for the transaction
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Private m_responseMap As New Dictionary

Private cat As New Logger

'****************************************************************************
'<comment>
' <summary>
'   Gets the transaction url property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get url() As String

    If m_url = "" Then
        url = capabilityUrls.Item(requestType)
        cat.debugLog ("getUrl():" + requestType + " url:" + url)
    Else
        url = m_url
    End If
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction url property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new url property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let url(aUrl As String)
    m_url = aUrl
End Property

'****************************************************************************
'<comment>
' <summary>Gets the transaction requestType property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get requestType() As String
    requestType = m_requestType
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction requestType property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new requestType property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let requestType(RHS As String)
    m_requestType = RHS
End Property

'****************************************************************************
'<comment>
' <summary>Gets the transaction context property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get context() As Dictionary
    Set context = m_context
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction context property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A Dictionary that is used as the new context property value.
' </param>
'</comment>
'****************************************************************************
Public Property Set context(RHS As Dictionary)
    Set m_context = RHS
End Property

'****************************************************************************
'<comment>
' <summary>Gets the transaction responseStatus property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get responseStatus() As String
    responseStatus = m_responseStatus
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction responseStatus property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new responseStatus property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let responseStatus(RHS As String)
    m_responseStatus = RHS
End Property

'****************************************************************************
'<comment>
' <summary>Gets the transaction responseStatusText property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get responseStatusText() As String
    responseStatusText = m_responseStatusText
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction responseStatusText property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new responseStatusText property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let responseStatusText(aResponseStatusText As String)
    m_responseStatusText = aResponseStatusText
End Property

'****************************************************************************
'<comment>
' <summary>Gets the transaction response property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get response() As String
    response = m_response
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction response property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A String that is used as the new response property value.
' </param>
'</comment>
'****************************************************************************
Public Property Let response(RHS As String)
    m_response = RHS
End Property

'****************************************************************************
'<comment>
' <summary>Gets the transaction requestMap property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get requestMap() As Dictionary
    Set requestMap = m_requestMap
End Property

'****************************************************************************
'<comment>
' <summary>Gets the transaction responseMap property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get responseMap() As Dictionary
    Set responseMap = m_responseMap
End Property

'****************************************************************************
'<comment>
' <summary>Sets the transaction responseMap property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A Dictionary that is used as the new responseMap property value.
' </param>
'</comment>
'****************************************************************************
Public Property Set responseMap(RHS As Dictionary)
    Set m_responseMap = RHS
End Property

'****************************************************************************
'<comment>
' <summary>
'   Preprocesses the transaction before the request is made to the RETS Server
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Sub Preprocess()
    ' does nothing
End Sub

'****************************************************************************
'<comment>
' <summary>
'   Postprocesses the transaction after the request is made to the RETS Server
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Sub Postprocess()
    ' does nothing
End Sub

'****************************************************************************
'<comment>
' <summary>Extracts the key-value pairs.
' </summary>
' <remarks></remarks>
' <param name="str">
'  A String that is parsed for the key-values.
' </param>
'</comment>
'****************************************************************************
Public Sub setKeyValuePairs(str As String)

    Dim equalSignPos As Integer
    Dim lines() As String
    Dim line
    Dim key As String
    Dim value As String
    
    If (str = "") Then
        Exit Sub
    End If

    lines = Split(str, vbLf)
    For Each line In lines

        ' if XML tag, ignore it
        If (Left(line, 1) <> "<") Then
            equalSignPos = InStr(line, "=")
            If (equalSignPos > 0) Then
                key = Trim(Left(line, equalSignPos - 1))
                value = Mid(line, equalSignPos + 1)
                
                ' if trailing CR remove
                If (Right(value, 1) = vbCr) Then
                    value = Left(value, Len(value) - 1)
                End If
                value = Trim(value)
                responseMap.Item(key) = value
            End If
        End If
    Next line
End Sub

'****************************************************************************
'<comment>
' <summary>Gets the transaction capabilityUrls property.
' </summary>
' <remarks></remarks>
'</comment>
'****************************************************************************
Public Property Get capabilityUrls() As Dictionary
    
    If (Not m_context.Exists(C_CAPABILITYURLS)) Then
        Set capabilityUrls = New Dictionary
        m_context.Add C_CAPABILITYURLS, capabilityUrls
    End If
    
    Set capabilityUrls = m_context.Item(C_CAPABILITYURLS)
    
End Property

'****************************************************************************
'<comment>
' <summary>Sets the capabilityUrls property to a new value.</summary>
' <remarks></remarks>
' <param name="RHS">
'  A Dictionary that is used as the new capabilityUrls property value.
' </param>
'</comment>
'****************************************************************************
Public Property Set capabilityUrls(RHS As Dictionary)
    If (m_context.Exists(C_CAPABILITYURLS)) Then
        m_context.Remove C_CAPABILITYURLS
    End If
    
    m_context.Add C_CAPABILITYURLS, RHS
End Property


