Attribute VB_Name = "modURL"
'****************************************************************************
'<comment>
' <summary>
'   Uniform Resource Locator (URL) Functions wrapping WININET functions.
' </summary>
' <remarks>
'   InternetCrackUrl include file For VBA
'   Created by Antonin Foller, http://www.pstruh.cz
' </remarks>
'</comment>
'****************************************************************************
Option Explicit
'*********************************** InternetCrackUrl
Enum eMaxWinInetValues
  INTERNET_MAX_HOST_NAME_LENGTH = 256
  INTERNET_MAX_USER_NAME_LENGTH = 128
  INTERNET_MAX_PASSWORD_LENGTH = 128
  INTERNET_MAX_PORT_NUMBER_LENGTH = 5          ' INTERNET_PORT is unsigned short
  INTERNET_MAX_PORT_NUMBER_VALUE = 65535       ' maximum unsigned short value
  INTERNET_MAX_PATH_LENGTH = 2048
  INTERNET_MAX_SCHEME_LENGTH = 32              ' longest protocol name length
  INTERNET_MAX_URL_LENGTH = INTERNET_MAX_SCHEME_LENGTH + 3 + INTERNET_MAX_PATH_LENGTH
End Enum

Public Type URL_COMPONENTSA
  dwStructSize As Long
  lpszScheme As String
  dwSchemeLength As Long
  nScheme As INTERNET_SCHEME
  lpszHostName As String
  dwHostNameLength As Long
  nPort As Integer
  
  lpszUsername As String
  dwUserNameLength As Long
  lpszPassword As String
  dwPasswordLength As Long
  
  lpszUrlPath As String
  dwUrlPathLength As Long
  lpszExtraInfo As String
  dwExtraInfoLength As Long
End Type

Enum eCrackURL
  ICU_ESCAPE = &H80000000       ' (un)escape URL characters
  ICU_DECODE = &H10000000       ' Convert %XX escape sequences To characters
End Enum

' flags for InternetCanonicalizeUrl() and InternetCombineUrl()
Enum eCanonizeURL
    ICU_NO_ENCODE = &H20000000    ' Don't convert unsafe characters to escape sequence
    ICU_DECODE = &H10000000       ' Convert %XX escape sequences to characters
    ICU_NO_META = &H8000000       ' Don't convert .. etc. meta path sequences
    ICU_ENCODE_SPACES_ONLY = &H4000000   ' Encode spaces only
    ICU_BROWSER_MODE = &H2000000  ' Special encode/decode rules for browser
End Enum

Enum INTERNET_SCHEME
    INTERNET_SCHEME_PARTIAL = -2
    INTERNET_SCHEME_UNKNOWN = -1
    INTERNET_SCHEME_DEFAULT = 0
    INTERNET_SCHEME_FTP
    INTERNET_SCHEME_GOPHER
    INTERNET_SCHEME_HTTP
    INTERNET_SCHEME_HTTPS
    INTERNET_SCHEME_FILE
    INTERNET_SCHEME_NEWS
    INTERNET_SCHEME_MAILTO
    INTERNET_SCHEME_SOCKS
    INTERNET_SCHEME_FIRST = INTERNET_SCHEME_FTP
    INTERNET_SCHEME_LAST = INTERNET_SCHEME_SOCKS
End Enum

Declare Function InternetCrackUrl Lib "WININET" Alias "InternetCrackUrlA" ( _
    ByVal lpszUrl As String, _
    ByVal dwUrlLength As Long, _
    ByVal dwFlags As eCrackURL, _
    lpUrlComponents As URL_COMPONENTSA) As Long
    
Declare Function InternetCanonicalizeUrl Lib "WININET" Alias "InternetCanonicalizeUrlA" ( _
    ByVal lpszUrl As String, _
    ByVal lpszBuffer As String, _
    ByVal lpdwBufferLength As Long, _
    ByVal dwFlags As eCanonizeURL) As Long
    
Declare Function InternetCombineUrl Lib "wininet.dll" Alias "InternetCombineUrlA" ( _
    ByVal lpszBaseUrl As String, _
    ByVal lpszRelativeUrl As String, _
    ByVal lpszBuffer As String, _
    lpdwBufferLength As Long, _
    ByVal dwFlags As eCanonizeURL) As Long


'****************************************************************************
'<comment>
' <summary>
'   Cracks a URL into its component parts.
' </summary>
' <remarks></remarks>
' <param name="url">
'  The string that contains the canonical URL to crack.
' </param>
'</comment>
'****************************************************************************
Public Function CrackURL(ByVal url As String) As URL_COMPONENTSA
  Dim c As URL_COMPONENTSA, Result As Long
  
  c.dwStructSize = 60
  c.lpszScheme = Space(INTERNET_MAX_SCHEME_LENGTH)
  c.dwSchemeLength = INTERNET_MAX_SCHEME_LENGTH
  c.lpszHostName = Space(INTERNET_MAX_HOST_NAME_LENGTH)
  c.dwHostNameLength = INTERNET_MAX_HOST_NAME_LENGTH
  
  c.lpszUsername = Space(INTERNET_MAX_USER_NAME_LENGTH)
  c.dwUserNameLength = INTERNET_MAX_USER_NAME_LENGTH
  c.lpszPassword = Space(INTERNET_MAX_USER_NAME_LENGTH)
  c.dwPasswordLength = INTERNET_MAX_USER_NAME_LENGTH
  
  c.lpszUrlPath = Space(INTERNET_MAX_PATH_LENGTH)
  c.dwUrlPathLength = INTERNET_MAX_PATH_LENGTH
  c.lpszExtraInfo = Space(128)
  c.dwExtraInfoLength = 128
  Result = InternetCrackUrl(url, 0, 0, c)
  If Result Then
    c.lpszScheme = Left$(c.lpszScheme, c.dwSchemeLength)
    c.lpszHostName = Left$(c.lpszHostName, c.dwHostNameLength)
    
    c.lpszUsername = Left$(c.lpszUsername, c.dwUserNameLength)
    c.lpszPassword = Left$(c.lpszPassword, c.dwPasswordLength)
    
    c.lpszUrlPath = Left$(c.lpszUrlPath, c.dwUrlPathLength)
    c.lpszExtraInfo = Left$(c.lpszExtraInfo, c.dwExtraInfoLength)
  Else
    c.lpszScheme = ""
    c.lpszHostName = ""
    
    c.lpszUsername = ""
    c.lpszPassword = ""
    
    c.lpszUrlPath = ""
    c.lpszExtraInfo = ""
  End If
  CrackURL = c
End Function

'****************************************************************************
'<comment>
' <summary>
'  Canonicalizes a URL, which includes converting unsafe
'       characters and spaces into escape sequences.
' </summary>
' <remarks></remarks>
' <param name="url">
'  The URL string that is to be Canonicalized
' </param>
' <param name="flags">
'  Flags that control canonicalization. Can be one of the following values:
'   ICU_BROWSER_MODE  Does not encode or decode characters after "#" or "?", and does not remove trailing white space after "?". If this value is not specified, the entire URL is encoded, and trailing white space is removed.
'   ICU_DECODE  Converts all %XX sequences to characters, including escape sequences, before the URL is parsed.
'   ICU_ENCODE_SPACES_ONLY  Encodes spaces only.
'   ICU_NO_ENCODE  Does not convert unsafe characters to escape sequences.
'   ICU_NO_META  Does not remove meta sequences (such as "." and "..") from the URL.
' </param>
'</comment>
'****************************************************************************
Function CanonicalizeUrl(url As String, flags As Long) As String
    Dim outUrl As String * 4096
    Dim outUrlLength As Long
    Dim Result As Integer
       
    Result = InternetCanonicalizeUrl(url, outUrl, outUrlLength, flags)
        
    If Result Then
        CanonicalizeUrl = Left(outUrl, outUrlLength)
    Else
        CanonicalizeUrl = vbNullString
    End If

End Function

